// Based on the excellent blog post:
// http://www.icab.de/blog/2010/01/12/search-and-highlight-text-in-uiwebview/

var FindInPageResultCount = 0;
var FindInPageResultMatches = [];
var FindInPagePreviousFocusMatchSpanId = null;

function recursivelyHighlightSearchTermInTextNodesStartingWithElement(element, searchTerm) {
    if (element) {
        if (element.nodeType == 3) {            // Text node
            while (true) {
                var value = element.nodeValue;  // Search for searchTerm in text node
                var idx = value.toLowerCase().indexOf(searchTerm);
                
                if (idx < 0) break;
                
                var span = document.createElement("span");
                var text = document.createTextNode(value.substr(idx, searchTerm.length));
                span.appendChild(text);
                span.setAttribute("class", "findInPageMatch");

                text = document.createTextNode(value.substr(idx + searchTerm.length));
                element.deleteData(idx, value.length - idx);
                var next = element.nextSibling;
                element.parentNode.insertBefore(span, next);
                element.parentNode.insertBefore(text, next);
                element = text;
                FindInPageResultCount++;
            }
        } else if (element.nodeType == 1) {     // Element node
            if (element.style.display != "none" && element.nodeName.toLowerCase() != "select") {
                for (var i = element.childNodes.length - 1; i >= 0; i--) {
                    recursivelyHighlightSearchTermInTextNodesStartingWithElement(element.childNodes[i], searchTerm);
                }
            }
        }
    }
}

function recursivelyRemoveSearchTermHighlightsStartingWithElement(element) {
    if (element) {
        if (element.nodeType == 1) {
            if (element.getAttribute("class") == "findInPageMatch") {
                var text = element.removeChild(element.firstChild);
                element.parentNode.insertBefore(text,element);
                element.parentNode.removeChild(element);
                return true;
            } else {
                var normalize = false;
                for (var i = element.childNodes.length - 1; i >= 0; i--) {
                    if (recursivelyRemoveSearchTermHighlightsStartingWithElement(element.childNodes[i])) {
                        normalize = true;
                    }
                }
                if (normalize) {
                    element.normalize();
                }
            }
        }
    }
    return false;
}

function deFocusPreviouslyFocusedSpan() {
    if(FindInPagePreviousFocusMatchSpanId){
        document.getElementById(FindInPagePreviousFocusMatchSpanId).classList.remove("findInPageMatch_Focus");
        FindInPagePreviousFocusMatchSpanId = null;
    }
}

function removeSearchTermHighlights() {
    FindInPageResultCount = 0;
    FindInPageResultMatches = [];
    deFocusPreviouslyFocusedSpan();
    recursivelyRemoveSearchTermHighlightsStartingWithElement(document.body);
}

function findAndHighlightAllMatchesForSearchTerm(searchTerm) {
    removeSearchTermHighlights();
    if (searchTerm.trim().length === 0){
        window.webkit.messageHandlers.findInPageMatchesFound.postMessage(FindInPageResultMatches);
        return;
    }
    searchTerm = searchTerm.trim();
    
    recursivelyHighlightSearchTermInTextNodesStartingWithElement(document.body, searchTerm.toLowerCase());
    
    // The recursion doesn't walk a first-to-last path, so it doesn't encounter the
    // matches in first-to-last order. We can work around this by adding the "id"
    // and building our results array *after* the recursion is done, thanks to
    // "getElementsByClassName".
    var orderedMatchElements = document.getElementsByClassName("findInPageMatch");
    FindInPageResultMatches.length = orderedMatchElements.length;
    for (var i = 0; i < orderedMatchElements.length; i++) {
        var matchSpanId = "findInPageMatchID|" + i;
        orderedMatchElements[i].setAttribute("id", matchSpanId);
        // For now our results message to native land will be just an array of match span ids.
        FindInPageResultMatches[i] = matchSpanId;
    }
    
    window.webkit.messageHandlers.findInPageMatchesFound.postMessage(FindInPageResultMatches);
}

function useFocusStyleForHighlightedSearchTermWithId(id) {
    deFocusPreviouslyFocusedSpan();
    setTimeout(function(){
        document.getElementById(id).classList.add("findInPageMatch_Focus");
        FindInPagePreviousFocusMatchSpanId = id;
    }, 0);
}

exports.findAndHighlightAllMatchesForSearchTerm = findAndHighlightAllMatchesForSearchTerm;
exports.useFocusStyleForHighlightedSearchTermWithId = useFocusStyleForHighlightedSearchTermWithId;
exports.removeSearchTermHighlights = removeSearchTermHighlights;
